@@ -22,6 +22,12 @@ try:
 except InvalidCacheBackendError:
     pass
 
+try:
+    import beaker.ext.google as google
+    clsmap['ext:google'] = google.GoogleContainer
+except (InvalidCacheBackendError, SyntaxError):
+    pass
+
 class Cache(object):
     """Front-end to the containment API implementing a data cache."""
     def __init__(self, namespace, **kwargs):
@@ -39,7 +45,7 @@ class Cache(object):
     set_value = put
     
     def get(self, key, **kwargs):
-        kw = self.kwargs
+        kw = self.kwargs.copy()
         kw.update(kwargs)
         kwargs.setdefault('type', 'memory')
         return self._get_container(key, **kw).get_value()
