@@ -28,7 +28,8 @@ class MemcachedNamespaceManager(NamespaceManager):
         
         verify_directory(self.lock_dir)            
         
-        self.mc = MemcachedNamespaceManager.clients.get(url, lambda: memcache.Client(url.split(';'), debug=0))
+        self.mc = MemcachedNamespaceManager.clients.get(url, 
+            lambda: memcache.Client(url.split(';'), debug=0))
 
     # memcached does its own locking.  override our own stuff
     def do_acquire_read_lock(self): pass
@@ -42,19 +43,20 @@ class MemcachedNamespaceManager(NamespaceManager):
     def close(self, *args, **params):pass
 
     def __getitem__(self, key):
-        key = key.replace(' ', '\302\267')
-        value = self.mc.get(self.namespace + "_" + key)
-        if value is None:
+        cache_key = key.replace(' ', '\302\267')
+        keys = [self.namespace + '_' + cache_key, self.namespace + ':keys']
+        key_dict = self.mc.get_multi(keys)
+        if cache_key not in key_dict.get(self.namespace+':keys', {}):
             raise KeyError(key)
-        return value
+        return key_dict[self.namespace + '_' + cache_key]
 
     def __contains__(self, key):
-        key = key.replace(' ', '\302\267')
-        return self.mc.get(self.namespace + "_" + key) is not None
+        return self.has_key(key)
 
     def has_key(self, key):
         key = key.replace(' ', '\302\267')
-        return self.mc.get(self.namespace + "_" + key) is not None
+        keys = self.mc.get(self.namespace + ':keys') or {}
+        return key in keys
 
     def __setitem__(self, key, value):
         key = key.replace(' ', '\302\267')
@@ -78,9 +80,9 @@ class MemcachedNamespaceManager(NamespaceManager):
     def do_remove(self):
         keys = self.mc.get(self.namespace + ':keys')
         if keys is not None:
-            for key in keys:
-                self.mc.delete(self.namespace + '_' + key)
-            self.mc.delete(self.namespace + ':keys')
+            delete_keys = [self.namespace + '_' + x for x in keys]
+            delete_keys.append(self.namespace + ':keys')
+            self.mc.delete_multi(delete_keys)
     
     def keys(self):
         keys = self.mc.get(self.namespace + ':keys')
